class ArgumentNotFoundError(Exception):
    pass


class ChannelNotFoundError(Exception):
    pass


class UserNotFoundError(Exception):
    pass


class ReactionNotFoundError(Exception):
    pass


class KeywordArgumentNotFoundError(Exception):
    pass


class MessageNotFoundError(Exception):
    pass


class KeyValueError(Exception):
    pass



from django.db import transaction
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from slackchat.serializers import MessageSerializer

from .celery import (
    create_private_channel,
    post_webhook,
    update_users,
    verify_webhook,
)
from .models import (
    Attachment,
    Channel,
    KeywordArgument,
    Message,
    Reaction,
    User,
    Webhook,
)


@receiver(post_save, sender=Channel)
def save_channel(sender, instance, created, **kwargs):
    if created:
        create_private_channel.delay(instance.pk)


@receiver(post_save, sender=Attachment)
@receiver(post_save, sender=Message)
@receiver(post_save, sender=Reaction)
@receiver(post_save, sender=KeywordArgument)
def notify_webhook(sender, instance, created, **kwargs):
    if sender == Message:
        channel_id = instance.channel.id.hex
        message = MessageSerializer(instance=instance).data
        chat_type = instance.channel.chat_type.name

        if created:
            update_type = "message_created"
        else:
            update_type = "message_changed"
    else:
        channel_id = instance.message.channel.id.hex
        message = MessageSerializer(instance=instance.message).data
        chat_type = instance.message.channel.chat_type.name
        update_type = "message_changed"

    transaction.on_commit(
        lambda: post_webhook.delay(channel_id, chat_type, update_type, message)
    )


@receiver(post_delete, sender=Attachment)
@receiver(post_delete, sender=Reaction)
@receiver(post_delete, sender=KeywordArgument)
def notify_webhook_deletion(sender, instance, **kwargs):
    channel_id = instance.message.channel.id.hex
    message = MessageSerializer(instance=instance.message).data
    chat_type = instance.message.channel.chat_type.name
    update_type = "message_changed"

    transaction.on_commit(
        lambda: post_webhook.delay(channel_id, chat_type, update_type, message)
    )


@receiver(post_delete, sender=Message)
def notify_webhook_message_delete(sender, instance, **kwargs):
    channel_id = instance.channel.id.hex
    message = MessageSerializer(instance=instance).data
    chat_type = instance.channel.chat_type.name
    update_type = "message_deleted"

    transaction.on_commit(
        lambda: post_webhook.delay(channel_id, chat_type, update_type, message)
    )


@receiver(post_save, sender=Webhook)
def save_webhook(sender, instance, **kwargs):
    verify_webhook.delay(instance.pk)


@receiver(post_save, sender=User)
def new_user(sender, instance, created, **kwargs):
    if created:
        update_users.delay([instance.pk])



# import json

# from django.conf import settings as project_settings
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from slackchat.conf import settings

from .handlers import (handle_message, handle_message_removed,
                       handle_reaction_added, handle_reaction_removed)


class Events(APIView):
    # Open API
    authentication_classes = ()
    permission_classes = ()

    def post(self, request, *args, **kwargs):
        slack_message = request.data

        # import json
        # print('DEBUG slackchat log:')
        # print(json.dumps(slack_message, indent=2))

        if slack_message.get('token') != settings.SLACK_VERIFICATION_TOKEN:
            return Response(status=status.HTTP_403_FORBIDDEN)

        if slack_message.get('type') == 'url_verification':
            return Response(
                data=slack_message.get('challenge'),
                status=status.HTTP_200_OK
            )

        if 'event' in slack_message:
            id = slack_message.get('event_id')
            event = slack_message.get('event')

            if event.get('type') == 'message':
                if event.get('subtype', None) == 'message_deleted':
                    handle_message_removed(id, event)
                else:
                    handle_message(id, event)

            if event.get('type') == 'reaction_added':
                handle_reaction_added(id, event)

            if event.get('type') == 'reaction_removed':
                handle_reaction_removed(id, event)

        return Response(status=status.HTTP_200_OK)



import uuid

from django.forms import widgets
from django.utils.safestring import mark_safe


class MarkdownEditorWidget(widgets.Textarea):
    def render(self, name, value, attrs=None):
        if 'class' not in attrs.keys():
            attrs['class'] = ''

        attrs['class'] += ' markdown-editor'

        attrs['data-uuid'] = str(uuid.uuid4())

        html = super(MarkdownEditorWidget, self).render(name, value, attrs)

        return mark_safe(html)

    class Media:
        js = (
            'https://cdnjs.cloudflare.com/ajax/libs/simplemde/1.11.2/simplemde.min.js',  # noqa: E501
            'slackchat/js/editor.js',
        )

        css = {
            'all': (
                'https://cdnjs.cloudflare.com/ajax/libs/simplemde/1.11.2/simplemde.min.css',  # noqa: E501
                'slackchat/css/editor.css',
            )
        }



# Generated by Django 2.0.2 on 2018-03-19 01:15

import uuid

import django.db.models.deletion
import django.utils.timezone
import slackchat.conf
import slackchat.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Argument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(help_text='The name of the argument that will be attached to a          message.', max_length=255)),
                ('character', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=300, null=True)),
                ('title_link', models.URLField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('service_name', models.CharField(blank=True, max_length=300, null=True)),
                ('service_icon', models.URLField(blank=True, null=True)),
                ('service_url', models.URLField(blank=True, null=True)),
                ('image_url', models.URLField(blank=True, null=True)),
                ('image_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('image_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('video_html', models.TextField(blank=True, null=True)),
                ('video_html_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('video_html_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('thumb_url', models.URLField(blank=True, null=True)),
                ('thumb_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('thumb_height', models.PositiveSmallIntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Channel',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('api_id', models.SlugField(blank=True, editable=False, help_text='Slack API channel ID', max_length=10, null=True)),
                ('team_id', models.SlugField(blank=True, editable=False, help_text='Slack API team ID', max_length=10, null=True)),
                ('title', models.CharField(blank=True, max_length=150, null=True)),
                ('introduction', slackchat.fields.MarkdownField(blank=True, help_text='Some introductory paragraph text (in markdown syntax).', null=True)),
                ('meta_title', models.CharField(blank=True, help_text='Page title', max_length=300, null=True)),
                ('meta_description', models.CharField(blank=True, help_text='Page description', max_length=300, null=True)),
                ('meta_image', models.URLField(blank=True, help_text='Share image URL', null=True)),
                ('publish_path', models.CharField(blank=True, help_text="A relative path you can use when         publishing the slackchat, e.g.,         <span style='color:grey; font-weight:bold;'>/2018-02-12/econ-chat/        </span>.", max_length=300, unique=True)),
                ('publish_time', models.DateTimeField(default=django.utils.timezone.now, help_text='Dateline.')),
                ('live', models.BooleanField(default=False, help_text='Determines whether page should re-poll for new messages         while chat is live.')),
            ],
        ),
        migrations.CreateModel(
            name='ChatType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(max_length=255)),
                ('publish_path', models.CharField(default='/slackchats/', help_text="A relative path for the slackchat type         you can append to any slackchat channel's         publish path, e.g.,         <span style='color:grey; font-weight:bold;'>/slackchats/        </span>.", max_length=300)),
                ('render_to_html', models.BooleanField(default=False, help_text='Whether to render markdown to HTML in the serializer.')),
                ('kwargs_in_threads', models.BooleanField(default=True, help_text='Whether users can create kwargs in threads.')),
            ],
        ),
        migrations.CreateModel(
            name='CustomContentTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('argument_name', models.SlugField(blank=True, help_text="Add an argument to the message if search_string         matches against a message's content.", max_length=255, null=True)),
                ('search_string', models.CharField(help_text='A regex search string with capture groups.', max_length=255)),
                ('content_template', models.TextField(help_text='A Python format string whose args are the capture groups         matched by the search_string.')),
                ('chat_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.ChatType')),
            ],
        ),
        migrations.CreateModel(
            name='KeywordArgument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('key', models.SlugField(max_length=30)),
                ('value', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('text', models.TextField()),
                ('channel', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='slackchat.Channel')),
            ],
        ),
        migrations.CreateModel(
            name='Reaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(unique=True)),
                ('reaction', models.CharField(max_length=150)),
                ('argument', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='slackchat.Argument')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='slackchat.Message')),
            ],
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_id', models.CharField(max_length=50)),
                ('first_name', models.CharField(blank=True, max_length=100, null=True)),
                ('last_name', models.CharField(blank=True, max_length=200, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('image', models.ImageField(upload_to=slackchat.conf.default_user_image_upload_to)),
                ('title', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('endpoint', models.URLField(unique=True)),
                ('verified', models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name='reaction',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='message',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='keywordargument',
            name='message',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='kwargs', to='slackchat.Message'),
        ),
        migrations.AddField(
            model_name='keywordargument',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='channel',
            name='chat_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='slackchat.ChatType'),
        ),
        migrations.AddField(
            model_name='channel',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='slackchat.User'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='message',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='slackchat.Message'),
        ),
        migrations.AddField(
            model_name='argument',
            name='chat_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='slackchat.ChatType'),
        ),
        migrations.AlterUniqueTogether(
            name='attachment',
            unique_together={('message', 'title_link', 'image_url')},
        ),
    ]



# Generated by Django 2.0.2 on 2018-04-14 21:44

import django.contrib.postgres.fields.jsonb
from django.db import migrations
import foreignform.fields


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='channel',
            name='extras',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='chattype',
            name='jsonSchema',
            field=foreignform.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='chattype',
            name='uiSchema',
            field=foreignform.fields.JSONField(blank=True, null=True),
        ),
    ]



import re
import uuid

import requests

from celery import shared_task
from slackchat.conf import settings
from slackchat.models import Webhook


@shared_task(acks_late=True)
def post_webhook(channel_id, chat_type, update_type=None, message=None):
    data = {
        "token": settings.WEBHOOK_VERIFICATION_TOKEN,
        "type": "update_notification",
        "channel": channel_id,
        "chat_type": chat_type,
        "update_type": None,
        "message": None,
    }

    if update_type:
        data["update_type"] = update_type

    if message:
        data["message"] = message

    for webhook in Webhook.objects.all():
        if webhook.verified:
            requests.post(webhook.endpoint, json=data)


@shared_task(acks_late=True)
def post_webhook_republish(channel_id, chat_type):
    data = {
        "token": settings.WEBHOOK_VERIFICATION_TOKEN,
        "type": "republish_request",
        "channel": channel_id,
        "chat_type": chat_type,
    }
    for webhook in Webhook.objects.all():
        if webhook.verified:
            requests.post(webhook.endpoint, json=data)


def clean_response(response):
    """ Cleans string quoting in response. """
    response = re.sub("^['\"]", "", response)
    response = re.sub("['\"]$", "", response)
    return response


@shared_task(acks_late=True)
def verify_webhook(pk):
    webhook = Webhook.objects.get(pk=pk)
    if not webhook.verified:
        challenge = uuid.uuid4().hex[:10]
        response = requests.post(
            webhook.endpoint,
            json={
                "token": settings.WEBHOOK_VERIFICATION_TOKEN,
                "type": "url_verification",
                "challenge": challenge,
            },
        )
        if response.status_code == requests.codes.ok:
            if clean_response(response.text) == challenge:
                webhook.verified = True
                webhook.save()
